from .db_engine_plugin_interface import DBPluginInterface
from .embed_interface import BaseEmbedding
from .loader_interface import BaseLoader
from .logger_interface import LoggerManagerInterface
from .retriever_interface import BaseRetriever
from .settings_interface import SettingsInterface
from .splitter_interface import BaseSplitter
from .task_engine_plugin_interface import TaskEnginPluginInterface

__all__ = [
    "DBPluginInterface",
    "BaseEmbedding",
    "BaseSplitter",
    "BaseRetriever",
    "BaseLoader",
    "LoggerManagerInterface",
    "SettingsInterface",
    "TaskEnginPluginInterface",
]



from abc import ABC, abstractmethod
from types import TracebackType
from typing import Any, Dict, List, Optional, Type, Union

import httpx
from pydantic import BaseModel


class BaseClient(ABC):
    def __init__(
        self,
        base_url: str,
        token: str,
        timeout: float = 10,
    ):
        self.base_url = base_url
        self.headers = {
            "Authorization": f"Bearer {token}",
            "Content-Type": "application/json",
        }
        self.timeout = timeout

    @abstractmethod
    async def _request(
        self,
        method: str,
        endpoint: str,
        *,
        json: Union[Dict[str, Any], List[Dict[str, Any]], None] = None,
        params: Optional[Union[Dict[str, Any], List[tuple[str, Any]]]] = None,
    ) -> Any:
        pass


class HttpClient(BaseClient):
    def __init__(
        self,
        base_url: str,
        token: str,
        timeout: float = 10,
    ):
        super().__init__(base_url, token, timeout)
        self.client = httpx.AsyncClient()

    async def __aenter__(self) -> "HttpClient":
        return self

    async def __aexit__(
        self,
        exc_type: Optional[Type[BaseException]],
        exc_val: Optional[BaseException],
        exc_tb: Optional[TracebackType],
    ) -> None:
        await self.close()

    async def _request(
        self,
        method: str,
        endpoint: str,
        *,
        json: Union[Dict[str, Any], List[Dict[str, Any]], None] = None,
        params: Optional[Union[Dict[str, Any], List[tuple[str, Any]]]] = None,
        **kwargs: Dict[str, Any],
    ) -> Any:
        url = f"{self.base_url}{endpoint}"
        request_kwargs = {
            "method": method,
            "url": url,
            "headers": self.headers,
            "params": params,
        }
        if json is not None:
            if isinstance(json, BaseModel):
                request_kwargs["json"] = json.model_dump(exclude_none=True)
            elif isinstance(json, (dict, list)):
                request_kwargs["json"] = json  # type: ignore
            else:
                raise ValueError(f"Unsupported JSON type: {type(json)}")
        for key, value in kwargs.items():
            if key not in request_kwargs:
                request_kwargs[key] = value

        response = await self.client.request(
            **request_kwargs, timeout=self.timeout  # type: ignore
        )
        response.raise_for_status()
        return response.json()

    async def close(self) -> None:
        if self.client:
            await self.client.aclose()



from datetime import datetime, timezone
from typing import Any, Optional
from uuid import UUID, uuid4

from pydantic import (
    BaseModel,
    ConfigDict,
    Field,
    field_serializer,
    field_validator,
    model_validator,
)

from whiskerrag_types.model.utils import parse_datetime


class Tenant(BaseModel):
    tenant_id: str = Field(
        default_factory=lambda: str(uuid4()), description="tenant id"
    )
    tenant_name: str = Field("", description="tenant name")
    email: str = Field(..., description="email")
    secret_key: str = Field("", description="secret_key")
    is_active: bool = Field(True, description="is active")
    created_at: Optional[datetime] = Field(
        default=None,
        description="tenant created time",
        alias="gmt_create",
    )
    updated_at: Optional[datetime] = Field(
        default=None,
        description="tenant updated time",
        alias="gmt_modified",
    )

    model_config = ConfigDict(
        populate_by_name=True,
    )

    def update(self, **kwargs: Any) -> "Tenant":
        for key, value in kwargs.items():
            setattr(self, key, value)

        self.updated_at = datetime.now()
        return self

    @field_validator("is_active", mode="before")
    @classmethod
    def convert_tinyint_to_bool(cls, v: Any) -> bool:
        return bool(v)

    @model_validator(mode="before")
    def pre_process_data(cls, data: dict) -> dict:
        for field, value in data.items():
            if isinstance(value, UUID):
                data[field] = str(value)
        field_mappings = {"created_at": "gmt_create", "updated_at": "gmt_modified"}
        for field, alias_name in field_mappings.items():
            val = data.get(field) or data.get(alias_name)
            if val is None:
                continue

            if isinstance(val, str):
                dt = parse_datetime(val)
                data[field] = dt
                data[alias_name] = dt
            else:
                if val and val.tzinfo is None:
                    dt = val.replace(tzinfo=timezone.utc)
                    data[field] = dt
                    data[alias_name] = dt

        return data

    @model_validator(mode="after")
    def set_defaults(self) -> "Tenant":
        now = datetime.now(timezone.utc)
        if self.created_at is None:
            self.created_at = now
        if self.updated_at is None:
            self.updated_at = now
        return self

    @field_serializer("created_at", "updated_at")
    def serialize_datetime(self, dt: datetime) -> str:
        return dt.strftime("%Y-%m-%dT%H:%M:%S.%fZ")



from abc import ABC, abstractmethod

from whiskerrag_types.model.knowledge import Knowledge


class BaseLoader(ABC):
    def __init__(self, knowledge: Knowledge):
        self.knowledge = knowledge

    @abstractmethod
    async def load(self) -> str:
        pass



from typing import List

from langchain_text_splitters import MarkdownTextSplitter

from whiskerrag_types.interface.splitter_interface import BaseSplitter
from whiskerrag_types.model.knowledge import KnowledgeTypeEnum, MarkdownSplitConfig
from whiskerrag_types.model.multi_modal import Text
from whiskerrag_utils.registry import RegisterTypeEnum, register


@register(RegisterTypeEnum.SPLITTER, KnowledgeTypeEnum.MARKDOWN)
class MarkdownSplitter(BaseSplitter[MarkdownSplitConfig, Text]):

    def split(self, content: str, split_config: MarkdownSplitConfig) -> List[str]:
        splitter = MarkdownTextSplitter(
            chunk_size=split_config.chunk_size,
            chunk_overlap=split_config.chunk_overlap,
        )
        return splitter.split_text(content)

    def batch_split(
        self, content: List[str], split_config: MarkdownSplitConfig
    ) -> List[List[str]]:
        return [self.split(text, split_config) for text in content]



from typing import List

from whiskerrag_types.model.chunk import Chunk
from whiskerrag_types.model.knowledge import Knowledge
from whiskerrag_types.model.multi_modal import Image

from .registry import (
    RegisterTypeEnum,
    RetrievalEnum,
    get_register,
    init_register,
    register,
)


async def get_chunks_by_knowledge(knowledge: Knowledge) -> List[Chunk]:
    LoaderCls = get_register(RegisterTypeEnum.KNOWLEDGE_LOADER, knowledge.source_type)
    SplitterCls = get_register(RegisterTypeEnum.SPLITTER, knowledge.knowledge_type)
    EmbeddingCls = get_register(
        RegisterTypeEnum.EMBEDDING, knowledge.embedding_model_name
    )
    content = await LoaderCls(knowledge).load()
    split_result = SplitterCls().split(content, knowledge.split_config)
    chunks = []
    for split_item in split_result:
        if isinstance(split_item, str):
            embedding = await EmbeddingCls().embed_text(split_item, timeout=30)
        elif isinstance(split_item, Image):
            embedding = await EmbeddingCls().embed_image(split_item, timeout=30)
        else:
            print(f"[warn]: illegal split item :{split_item}")
            continue
        chunk = Chunk(
            context=split_item if isinstance(split_item, str) else str(split_item),
            metadata={},
            embedding=embedding,
            knowledge_id=knowledge.knowledge_id,
            embedding_model_name=knowledge.embedding_model_name,
            space_id=knowledge.space_id,
            tenant_id=knowledge.tenant_id,
        )
        chunks.append(chunk)
    return chunks


__all__ = [
    "get_register",
    "register",
    "RegisterTypeEnum",
    "init_register",
    "RetrievalEnum",
    "SplitterEnum",
    "get_chunks_by_knowledge",
]
